//-------------------------------------------------------------------------------
// RadioWaves.class - A Java applet that draws radio waves emanating from
//                    its centre.
//-------------------------------------------------------------------------------
// Copyright (C) 1997 by James MacKay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The author can be contacted via e-mail at: jmackay@sentex.net
//--------------------------------------------------------------------------
import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Rectangle;

/**
* Draws repeating series of radio waves emitted from its centre.
* Also see RadioWaves` <a href="RadioWaves-params.html">applet
* parameters documentation</a>.
* @version 1.1
* @author James MacKay (<a href="mailto:jmackay@sentex.net">jmackay@sentex.net</a>)
*/
public class RadioWaves extends Applet implements Runnable
   {
   private int centreX, centreY;
   private int initialRadius, maxRadius;
   private int radiusIncrement;
   private int width, height;
   private Thread radio;
   private Color waveColor;
   private int waveDelay, seriesDelay;
   private boolean clearBetweenSeries;
   private int numSeries;

   public void init()
      {
      radio = null;

      Rectangle b = bounds();
      width  = b.width;
      height = b.height;
      centreX = width / 2;
      centreY = height / 2;
      maxRadius = Math.max(height / 2, width / 2);

      initialRadius      = getIntParameter("initialRadius", 0);
      radiusIncrement    = getIntParameter("radiusIncr", 1);
      waveDelay          = getIntParameter("waveDelay", 0);
      seriesDelay        = getIntParameter("seriesDelay", 0);
      waveColor          = getColorParameter("waveColor", Color.black);
      clearBetweenSeries = getBooleanParameter("clearBetweenSeries", true);
      numSeries          = getIntParameter("seriesCount", -1);
      }

   public void start()
      {
      if (radio == null)
         {
         radio = new Thread(this);
         radio.start();
         }
      }

   public void stop()
      {
      if ( (radio != null) && radio.isAlive() )
         radio.stop();
      radio = null;
      }

   /**
   * Repeatedly emits series of radio waves.
   */
   public void run()
      {
      Graphics g = this.getGraphics();
      int currSeries = 0;
      while (numSeries < 0 || currSeries++ < numSeries)
         emitWaveSeries(g);
      }

   /**
   * Emits a series of radio waves.
   * @param g the Graphics object on which to draw the waves
   */
   private void emitWaveSeries(Graphics g)
      {
      initWaveColors(g);
      for (int radius = 0; radius >= 0 && radius <= maxRadius;
           radius += radiusIncrement)
         {
         setNextWaveColor(g);
         g.drawOval(centreX - radius, centreY - radius,
                    2 * radius, 2 * radius);
         if (waveDelay > 0)
            {
            try Thread.sleep(waveDelay);
            catch (InterruptedException e);
            }
         }  // for

      if (clearBetweenSeries)
         g.clearRect(0, 0, width, height);
      if (seriesDelay > 0)
         {
         try Thread.sleep(seriesDelay);
         catch (InterruptedException e);
         }
      }  // emitWaveSeries(Graphics)

   /**
   * Initializes the wave colour(s) before a series of waves is emitted.
   * @param g the Graphics object on which the series of waves will be drawn
   */
   protected void initWaveColors(Graphics g)
      {
      g.setColor(waveColor);
      }

   /**
   * Sets the colour of the next wave to be drawn.
   * @param g the Graphics object on which the wave will be drawn
   */
   protected void setNextWaveColor(Graphics g)
      {
      // empty
      }

   /**
   * Returns version, author and copyright information about the
   * RadioWaves applet.
   */
   public String getAppletInfo()
      {
      return "RadioWaves version 1.1, Copyright (C) 1997 by James MacKay";
      }

   /**
   * Describes each of RadioWaves` applet parameters.
   */
   public String[][] getParameterInfo()
      {
      String[][] info =
         {
         { "initialRadius", "number of pixels",
              "radius of the first wave in each series (default is 0)" },
         { "radiusIncr", "number of pixels",
              "difference between the radii of successive waves " +
               "(default is 1)"},
         { "waveColor", "hexadecimal colour value",
              "colour of the waves (default is black)" },
         { "waveDelay", "number of milliseconds",
              "length of pause between waves (default is 0)" },
         { "seriesDelay", "number of milliseconds",
              "length of pause between series of waves (default is 0)" },
         { "clearBetweenSeries", "boolean",
              "whether to erase all waves between series " +
               "(default is true)"},
         { "seriesCount", "count",
              "number of series of waves (infinite if negative, " +
              "default is -1)"}
         };  // String[][] info

      return info;
      }

   /**
   * Returns the integer value of the applet parameter <var>paramName</var>,
   * or <var>defaultValue</var> if the parameter isn`t specified or isn`t
   * an integer.
   * @param paramName the name of the applet parameter whose value is to
   *                  be returned
   * @param defaultValue the value to be returned if the applet parameter
   *                     hasn`t been specified or isn`t an integer
   */
   protected final int getIntParameter(String paramName, int defaultValue)
      {
      String strValue = getParameter(paramName);
      try
         return Integer.parseInt(strValue, 10);
      catch (NumberFormatException e) 
         return defaultValue;
      }

   /**
   * Returns the boolean value of the applet parameter <var>paramName</var>,
   * or <var>defaultValue</var> if the parameter isn`t specified or isn`t
   * a boolean ("true" or "false").
   * @param paramName the name of the applet parameter whose value is to
   *                  be returned
   * @param defaultValue the value to be returned if the applet parameter
   *                     hasn`t been specified or isn`t a boolean
   */
   protected final boolean getBooleanParameter(String paramName,
                                               boolean defaultValue)
      {
      String strValue = getParameter(paramName);
      if (strValue == null)
         return defaultValue;
      else if (strValue.equalsIgnoreCase("true"))
         return true;
      else if (strValue.equalsIgnoreCase("false"))
         return false;
      return defaultValue;
      }

   /**
   * Returns the Color specified by the value of the applet parameter
   * <var>paramName</var>, or <var>defaultValue</var> if the parameter
   * isn`t specified or isn`t valid. (The parameter`s value must be a
   * hexadecimal colour value of the form RRGGBB.)
   * @param paramName the name of the applet parameter whose value is to
   *                  be returned
   * @param defaultValue the value to be returned if the applet parameter
   *                     hasn`t been specified or isn`t valid
   */
   protected final Color getColorParameter(String paramName,
                                           Color defaultValue)
      {
      String strValue = getParameter(paramName);
      int colorNumber;
      try
         colorNumber = Integer.parseInt(strValue, 16);
      catch (NumberFormatException e)
         return defaultValue;

      if (colorNumber < 0 || colorNumber > 0xffffff)
         return defaultValue;
      return new Color(colorNumber);
      }
   } // class RadioWaves

